#!/usr/bin/env python
"""File for automising the translation process.
Run this when you need updated .po and .pot files.
Run this when you need updated .mo files.
Add languages by modifying the languages variable.
Automatically adds new files to bzr, unless --ignore-bzr is specified.
NOTE: you will need to install intltool and po4a for this to work.
"""

import os, sys
import subprocess
from xml.dom import minidom

# All supported languages should be added here and in ../po4a.cfg
languages = ("en_GB", "pt_BR", "it", "es")

def main():
    if '--help' in sys.argv[1:]:
        usage()
        sys.exit(0)
    print "languages: %s" % " ".join(languages)
    gpot, gpo, gmo, addBzr = (False, False, False, True)
    if '--generate-pot' in sys.argv[1:]:
        gpot = True
    if '--generate-po' in sys.argv[1:]:
        gpo = True
    if '--generate-mo' in sys.argv[1:]: # this also includes docbook documentation
        gmo = True
    if not True in (gpot, gpo, gmo):
        print "Defaulting to --generate-pot --generate-po"
        gpot, gpo, gmo = (True, True, False)
    if "--ignore-bzr" in sys.argv[1:]:
        addBzr = False
    
    dp = determine_path()
    pp = os.path.join(dp[:-1*len(dp.split("/")[-1])], "epidermis")
    rp = dp[:-1*len(dp.split("/")[-1])]
    # pp should be the epidermis dir with all the .py files in it
    # rp is the root directory of the branch
    
    poDir = os.path.join(dp[:-1*len(dp.split("/")[-1])], "po")
    
    try:
        sanity_check(os.path.join(poDir,"epidermis.pot"))
    except SanityCheckFailed, ee:
        print  "Warning: %s\n" % ee.__str__()
    for lang in languages:
        try:
            sanity_check(os.path.join(poDir,lang + ".po"))
        except SanityCheckFailed, ee:
            print "Warning: %s" % ee.__str__()
    
    sys.path.insert(1, rp)
    import epidermis.const
    VERSION = epidermis.const.VERSION
    
    if gpot:
        # make pot file
        
        call("mkdir -p %s" % poDir, shell=True)
        
        call(("xgettext --language=Python --keyword=_ --keyword=_n:1,2 --from-code=UTF-8 " + \
            "--package-name=Epidermis --package-version=%s " + \
            "--msgid-bugs-address=https://bugs.launchpad.net/epidermis " + \
            "--output=%s/epidermis.pot *.py pigments/*.py") % (VERSION, poDir),
            shell=True, cwd=pp)
        
        # add glade files
        call("xgettext --language=Glade --from-code=UTF-8 --output=glade.pot *.glade", shell=True, cwd=os.path.join(rp,"data"))
        call("msgcat --use-first po/epidermis.pot data/glade.pot -o po/epidermis.pot",shell=True,cwd=rp)
            
        # add documentation to pot file
        call("po4a-gettextize -f docbook -m help/C/epidermis-doc.xml -p help/epidermis-doc.pot", shell=True, cwd=rp)
        call("msgcat --use-first po/epidermis.pot help/epidermis-doc.pot -o po/epidermis.pot", shell=True, cwd=rp)
        call("po4a-gettextize -f man -m linux/epidermis.1 -o groff_code=verbatim -p linux/epidermis-man.pot", shell=True, cwd=rp)
        call("msgcat --use-first po/epidermis.pot linux/epidermis-man.pot -o po/epidermis.pot", shell=True, cwd=rp)
        call("po4a-gettextize -f xml -o \"translated=<title> <description>\" -o \"untranslated=<creator> <date> <type>\" -o \"attributes=<language>code <identifier>url\" -m help/C/epidermis-C.omf -p help/epidermis-omf.pot", shell=True, cwd=rp)
        call("msgcat --use-first po/epidermis.pot help/epidermis-omf.pot -o po/epidermis.pot", shell=True, cwd=rp)
        call("rm help/epidermis-doc.pot linux/epidermis-man.pot help/epidermis-omf.pot data/glade.pot", shell=True, cwd=rp)
        
        # add translatable strings from .policy configuration file
        doc = minidom.parse(os.path.join(rp, "linux/org.tuxfamily.epidermis.shell.policy"))
        input = ""
        for action in doc.getElementsByTagName("action"):
            for tagname in ("description", "message"):
                tags = action.getElementsByTagName(tagname)
                tags.reverse()
                for tag in tags:
                    if not tag.getAttribute("xml:lang"):
                        break # we've found the untranslated tag
                input += "_(" + repr(get_first_text(tag)) + ")\n"
        subprocess.call(["mkdir","-p","/tmp/epidermis-extra"])
        open("/tmp/epidermis-extra/org.tuxfamily.epidermis.shell.policy","w").write(input)
        subprocess.call("xgettext --language=Python --keyword=_ --from-code=UTF-8 "
            "--output=/tmp/epidermis-extra.pot org.tuxfamily.epidermis.shell.policy",
            cwd="/tmp/epidermis-extra", shell=True)
        subprocess.call("msgcat --use-first po/epidermis.pot /tmp/epidermis-extra.pot "
            "-o po/epidermis.pot", shell=True, cwd=rp)
        
        # add translatable strings from .desktop file
        input = ""
        for line in open(os.path.join(rp,"linux/epidermis.desktop"),"r").readlines():
            for item in ("Name", "Comment"):
                if line.startswith(item + "="):
                    input += "_(" + repr(line.split("=")[1][:-1]) + ")\n"
        open("/tmp/epidermis-extra/epidermis.desktop","w").write(input)
        subprocess.call("xgettext --language=Python --keyword=_ --from-code=UTF-8 "
            "--output=/tmp/epidermis-extra.pot epidermis.desktop ",
            cwd="/tmp/epidermis-extra", shell=True)
        subprocess.call("msgcat --use-first po/epidermis.pot /tmp/epidermis-extra.pot "
            "-o po/epidermis.pot", shell=True, cwd=rp)
        
        # bzr add the final pot file
        if addBzr: call("bzr add %s/epidermis.pot" % poDir, shell=True, cwd=pp)
        
        sanity_check(os.path.join(poDir, "epidermis.pot"))
    
    if gmo:
        # find where we want the built files to be
        build_dir = os.environ.get("MO_BUILD_DIR",rp)
        call(["mkdir","-p","help","linux","po"],cwd=build_dir)
    
    for lang in languages:
        if gpo and not os.path.exists(os.path.join(poDir, lang + ".po")):
            # note: cwd is poDir here
            call("msginit --input=epidermis.pot --locale=%s --no-translator" % lang, shell=True, cwd=poDir)
            if addBzr: call("bzr add %s/%s.po" % (poDir, lang), shell=True, cwd=pp)
        elif gpo:
            # note: cwd is poDir here
            call("msgmerge -U %s.po epidermis.pot" % lang, shell=True, cwd=poDir)
        # make .mo files
        if gmo:
            call("mkdir -p %s" % os.path.join(build_dir,"po",lang,"LC_MESSAGES"), shell=True, cwd=pp)
            call("msgfmt -o %s %s/%s.po" % (os.path.join(build_dir,"po",lang,
                "LC_MESSAGES/epidermis.mo"),poDir, lang), shell=True)
            call("cp po/%s.po help/%s.po" % (lang,lang), shell=True, cwd=rp)
        sanity_check(os.path.join(poDir,lang + ".po"))
    if gmo:
        # check that po4a.cfg includes all the languages
        first_line = open(os.path.join(rp,"po4a.cfg"),"r").readlines()[0]
        for lang in languages:
            if lang not in first_line:
                raise Exception("%s not found in first line of po4a.cfg" % lang)
        # generate docbooks and man pages using po4a
        call("cp -r po po4a", shell=True, cwd=rp)
        call("po4a po4a.cfg", shell=True, cwd=rp)
        
        for lang in languages:
            # move generated docbooks and man pages to build_dir
            if not os.path.samefile(rp, build_dir):
                call(["rm", "-rf", os.path.join(build_dir, "help", lang)])
                call(["mv", "help/%s" % lang, os.path.join(build_dir, "help")], cwd=rp)
                call(["mv", "linux/epidermis.%s.1" % lang, os.path.join(build_dir,"linux")], cwd=rp)
            # copy documentation screenshots over
            if os.path.exists(os.path.join(rp, "po/help/%s/images" % lang)):
                call("cp -rT po/help/%s/images %s/help/%s/images" % (lang, build_dir,lang), shell=True, cwd=rp)
            else:
                print "WARNING: copying english screenshots to %s translation" % lang
                call("cp -rT help/C/images %s/help/%s/images" % (build_dir,lang), shell=True, cwd=rp)
            call("rm help/%s.po" % lang, shell=True, cwd=rp)
            # modify epidermis-lang.omf so that it points to the correct documentatian
            omf = minidom.parse(os.path.join(build_dir, "help/%s/epidermis-%s.omf" % (lang,lang)))
            omf.getElementsByTagName("identifier")[0].setAttribute("url","file:///usr/share/gnome/help/epidermis/%s/epidermis-doc.xml" % lang)
            omf.getElementsByTagName("language")[0].setAttribute("code", lang)
            ff = open(os.path.join(build_dir, "help/%s/epidermis-%s.omf" % (lang,lang)), "w")
            ff.write(omf.toxml().encode('utf-8'))
            ff.close()
            del(omf, ff)
        
        call("rm -r po4a", shell=True, cwd=rp)
        print "generated docbooks and man pages"

def sanity_check(path):
    """Make sure that the .pot or .po file specified in path has been
    correctly compiled"""
    if not os.path.exists(path):
        return
    ff = open(path,"r")
    necessary_lines = {"#: epidermis.py":False, "#: creator.py":False,
        "#: creator.glade":False, "#: epidermis.glade":False,
        "#: help/C/epidermis-doc.xml":False, "#: linux/epidermis.1":False,
        "#: help/C/epidermis-C.omf":False,
        "#: org.tuxfamily.epidermis.shell.policy":False,
        "#: epidermis.desktop": False}
    foundRemoved = False
    for line in ff.readlines():
        for necessary_line in necessary_lines.keys():
            if necessary_line in line and line.index(necessary_line) == 0:
                necessary_lines[necessary_line] = True
        if line[:2] == "#~":
            foundRemoved = True
    if foundRemoved:
        print >> sys.stderr, "Warning: found removed lines in %s" % path
    ff.close()
    for necessary_line in necessary_lines.keys():
        if not necessary_lines[necessary_line]:
            raise SanityCheckFailed(path,necessary_lines)
    return True

class SanityCheckFailed(Exception):
    def __init__(self, path, necessary_lines):
        self.path = path
        self.necessary_lines = necessary_lines
    def __str__(self):
        return "sanity_check failed for %s, necessary_lines: %s" \
            % (self.path, str(self.necessary_lines))

def get_first_text(thexml):
    """Get text of XML node (ignores comments)"""
    import xml.dom
    for el in thexml.childNodes:
        if el.nodeType == xml.dom.Node.TEXT_NODE:
            return el.data
    raise None

def call(*args, **kwargs):
    """a wrapper round subprocess.call, raises an Exception when the command returns
    a non-null error value"""
    rv = subprocess.call(*args, **kwargs)
    if rv != 0:
        class BadCall(Exception):
            def __init__(self, args, kwargs):
                self.args, self.kwargs = args, kwargs
            def __str__(self):
                return "BadCall; args: %s kwargs: %s" % (str(args), str(kwargs))
        raise(BadCall(args,kwargs))

def usage():
    """print help text about the script"""
    print """translate.py
--generate-pot: create gettext portable object template file epidermis.pot
--generate-po: create gettext portable objects (.po) for all supported languages
--generate-mo: create gettext binary portable objects (.mo) for all supported languages
If none of the above options are specified, the first two will be used. However, if one
or more of the options are specified, then the others won't be.
--help: display this message
--ignore-bzr: do not add new generated files to the bazaar branch"""
        
    
def determine_path ():
    """Borrowed from wxglade.py"""
    try:
        root = __file__
        if os.path.islink (root):
            root = os.path.realpath (root)
        return os.path.dirname (os.path.abspath (root))
    except Exception, ee:
        print >> sys.stderr, "I'm sorry, but something is wrong."
        print >> sys.stderr, "There is no __file__ variable. Please contact the author."
        sys.exit (1)
        raise(ee)

if __name__ == "__main__":
    main()
