#!/usr/bin/env python2.6
# Copyright 2008-2010 David D Lowe
# Released under the GPLv3
"""Module with constant variables that are used accross several modules
in the Epidermis project

Import at the beginning of the module like this:
>>> from const import *
>>> if __name__ == "__main__":
...    set_up_const()
...    from const import *
...

The set_up_const function must be called for values like USERNAME to be correct"""

import os, sys, subprocess
import logging
import getpass

# Pigment directory shared between all users
SHARED_PIGMENT_DIR = "/usr/share/epidermis/pigments/"

# List of all recognised pigment types
PIGMENT_TYPES = ["skin", "wallpaper", "metacity", "gtk", "icons", "gnomeSplash", \
    "cursors", "grub", "gdm", "xsplashHack"]

# Are we in debugging mode?
DEBUG = False
if '--debug' in sys.argv[1:]:
    DEBUG = True

# set up the logger "default"
logger = logging.getLogger("default")
if DEBUG:
    logger.setLevel(logging.DEBUG)
__stream = logging.StreamHandler()
__stream.setFormatter(logging.Formatter("%(levelname)s: %(message)s " + \
    "(File \"%(pathname)s\", line %(lineno)d, in %(funcName)s)"))
logger.addHandler(__stream)
del(__stream)

# The name of the thread in which gtk.main() is running
GTK_LOOP_THREAD_NAME = "GtkThread"

# Some strings
CUSTOM = "CUSTOM: " # remember to surround this string with _()
APP_NAME = "epidermis"
LOCALE_DIR = "po"
VERSION = "0.5.1"
VERSION_TAG = ""

# other constonants
PREVIEW_WIDTH = 100
PREVIEW_HEIGHT = 75

const_initialised = False

# Get current user's username
USERNAME = getpass.getuser()

# PAR_DIR is the parent directory of the epidermis python package
PAR_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Check whether Epidermis is installed or running from source
INSTALLED = True
try:
    if os.path.exists(os.path.join(PAR_DIR, "setup.py")) and os.path.isdir(
    os.path.join(PAR_DIR,"data")):
        INSTALLED = False
except:
    pass
   

# Cache directory for current user, MY_CACHE_HOME is usually ~/.cache/epidermis
if os.environ.has_key('XDG_CACHE_HOME'):
    CACHE_HOME = os.environ['XDG_CACHE_HOME']
else:
    if os.environ.has_key('HOME'):
        CACHE_HOME = os.path.join(os.environ['HOME'],".cache")
        MY_CACHE_HOME = os.path.join(CACHE_HOME, "epidermis")
    else:
        CACHE_HOME = None
# Standards: http://standards.freedesktop.org/basedir-spec/latest/ar01s03.html
if os.environ.has_key('XDG_CONFIG_HOME'):
    CONFIG_HOME = os.environ['XDG_CONFIG_HOME']
else:
    if os.environ.has_key('HOME'):
        CONFIG_HOME = os.path.join(os.environ['HOME'], ".config")
        MY_CONFIG_HOME = os.path.join(CONFIG_HOME, "epidermis/")
    else:
        CONFIG_HOME = None


if os.environ.has_key('XDG_DATA_HOME'):
    DATA_HOME = os.environ['XDG_DATA_HOME']
else:
    if os.environ.has_key('HOME'):
        DATA_HOME = os.path.join(os.environ['HOME'],".local/share")
        MY_DATA_HOME = os.path.join(DATA_HOME, "epidermis/")
    else:
        DATA_HOME = None

const_initialized = True

if __name__ == "__main__":
    print >> sys.stderr, "Don't run"
    sys.exit(1)

